/*********************************************************************************************/
/* purpose: Estimate and confidence interval of ICC using ANOVA method.                      */
/* In original data, there are                                                               */
/*   3 raters(coded as 1,2 and 3) and 2 occasions (coded as 1 and 2).  Each rater evaluate a */
/*   sample of 30 radiography films at each occasion.  There are 180 observations altogether.*/
/*   For the meaning of the measurements, refer to the paper -- "Estimation of reliability   */
/*   in a three-factor model".                                                               */
/*********************************************************************************************/
DM editor 'clear log' editor; DM editor 'clear output' editor;

OPTIONS LS=96;

/* Specify where the original data is save, user of this program need to change the directory */
%LET Location=C:\Documents and Settings\etc\;
LIBNAME sto "&location";

PROC FORMAT;
  INVALUE  num  'A'=1 'B'=2 'C'=3 '1'=1 '2'=2;
RUN;

/* Read in data regarding measurement RRAVT1T2 */
DATA Xray; 
  SET sto.Xray_RRAVT1T2;
RUN;

PROC TRANSPOSE DATA=Xray OUT=Adata;
  BY MEASURE FILM;
RUN;

DATA Adata;SET Adata;
	RATER=input(SUBSTR(_NAME_,1,1), num.);
	OCCAS=input(SUBSTR(_NAME_,2,1), num.);
	RENAME COL1=x;
	DROP _NAME_;
RUN;

/* Model */
PROC GLM DATA=Adata OUTSTAT=STAT noprint; 
  BY MEASURE;
  CLASS FILM RATER OCCAS;
  MODEL x=FILM RATER OCCAS FILM*RATER RATER*OCCAS FILM*OCCAS/SS3;
RUN;

/* Calculate Mean Square due to each factor */
DATA MS;SET STAT;
  MS=SS/DF;
  KEEP MEASURE _SOURCE_ MS;
RUN;

/* Degrees of freedom of each factor */
DATA DF;SET STAT;
  KEEP MEASURE _SOURCE_ DF;
RUN;

PROC TRANSPOSE DATA=MS OUT=MS1;
  BY MEASURE;
  ID _SOURCE_;
  VAR MS;
RUN;

PROC TRANSPOSE DATA=DF OUT=DF1 PREFIX=DF;
  BY MEASURE;
  ID _SOURCE_;
  VAR DF;
RUN;

/* rename the means squares and degrees of freedom according to the symbols used in paper */
DATA MS_DF;
   SET MS1;
   SET DF1;
   rename ERROR= MS_e; rename FILM=MS_p; rename RATER=MS_r; rename OCCAS=MS_o;
   rename FILM_RATER=MS_pr; rename RATER_OCCAS=MS_ro; rename FILM_OCCAS=MS_po;

   rename DFERROR=DF_e; rename DFFILM=DF_p; rename DFRATER=DF_r; rename DFOCCAS=DF_o;
   rename DFFILM_RATER=DF_pr; rename DFRATER_OCCAS=DF_ro; rename DFFILM_OCCAS=DF_po;

   n_p=DFFILM+1; n_r=DFRATER+1; n_o=DFOCCAS+1; 
RUN;

/* Calculation estimate and CI of ICC*/
DATA ICC; SET MS_DF;
  numer1=(MS_p+MS_e-MS_pr-MS_po)/(n_r*n_o);
  denom1=MS_p/(n_r*n_o)+MS_r/(n_p*n_o)+MS_o/(n_p*n_r)+MS_pr*(1/n_o-1/(n_r*n_o)-1/(n_p*n_o))
       +MS_po*(1/n_r-1/(n_r*n_o)-1/(n_p*n_r))+MS_ro*(1/n_p-1/(n_p*n_o)-1/(n_p*n_r))
	   +MS_e*(1+1/(n_r*n_o)+1/(n_p*n_o)+1/(n_p*n_r)-1/n_p-1/n_r-1/n_o);
  ICC=numer1/denom1;         /* estimate of ICC*/

  ICCR=ICC/(1-ICC);
  e=(n_r/n_p)*ICCR;
  f=(n_o/n_p)*ICCR;
  g=1+n_r*ICCR-(n_r/n_p)*ICCR;
  h=1+n_o*ICCR-(n_o/n_p)*ICCR;
  i=(n_r*n_o-n_r-n_o)/n_p*ICCR;
  j=(n_r/n_p+n_o/n_p-n_r-n_o-n_r*n_o/n_p+n_r*n_o)*ICCR-1;

  numer2=(e*MS_r+f*MS_o+g*MS_pr+h*MS_po+i*MS_ro+j*MS_e)**2;
  denom2=(e*MS_r)**2/DF_r+(f*MS_o)**2/DF_o+(g*MS_pr)**2/DF_pr
         +(h*MS_po)**2/DF_po+(i*MS_ro)**2/DF_ro+(j*MS_e)**2/DF_e;

  v=ROUND(numer2/denom2,1);  /* denominator df */
  q=(n_r/n_p)*MS_r+(n_o/n_p)*MS_o+(n_r-n_r/n_p)*MS_pr+(n_o-n_o/n_p)*MS_po
     +(n_r*n_o-n_r-n_o)/n_p*MS_ro+(n_r+n_o-n_r*n_p-n_o*n_p-n_r*n_o+n_p*n_r*n_o)/n_p*MS_e;


  E025 = FINV(0.025,DF_p,v);
  E975 = FINV(0.975,DF_p,v);

  L_limit = (MS_p-E975*(MS_pr+MS_po-MS_e))/(E975*(q-MS_pr-MS_po+MS_e)+MS_p); /* upper limit of ICC */
  U_limit = (MS_p-E025*(MS_pr+MS_po-MS_e))/(E025*(q-MS_pr-MS_po+MS_e)+MS_p); /* lower limit of ICC */

  Length = U_limit-L_limit;
RUN;

/* Where the estimate and CI of ICC are saved */
PROC PRINT DATA=ICC;
  TITLE "The estimated Intra-class Correlation Coefficiant and confidence interval";
  VAR MEASURE ICC L_limit U_limit Length;
RUN;
