/*********************************************************************************************/
/* purpose: Calculate estimated and confidence interval of ICC using bootstrapping method.   */
/*          Efron, Hall and Bias-corrected percentile methods are applied.  But only Efron   */
/*          and Bias-corrected results are output.                                           */
/* The "boot" is the main macro to fulfill this task.                                        */ 
/* Argument:                                                                                 */
/*          Odata is the data need to be analyzed;*/
/*          Nsamples is the number of resamples taken from orginal data;                     */
/*          random is the seed for uniform bootstrap resampling.                             */
/*********************************************************************************************/
DM editor 'clear log' editor; DM editor 'clear output' editor;

%LET Location=C:\Documents and Settings\etc\;
LIBNAME sto "&location";

%Macro analyze(Odata=, out=, by=);
PROC FORMAT;
  INVALUE  num  'A'=1 'B'=2 'C'=3 '1'=1 '2'=2;
RUN;

%IF &by= %THEN %DO;
PROC TRANSPOSE DATA=&Odata OUT=Adata;
  BY &by MEASURE FILM;
RUN;
%END;
%ELSE %DO;
PROC TRANSPOSE DATA=&Odata OUT=Adata;
  BY &by _i MEASURE FILM;
RUN;
%END;

DATA Adata;SET Adata;
	RATER=input(SUBSTR(_NAME_,1,1), num.);
	OCCAS=input(SUBSTR(_NAME_,2,1), num.);
	RENAME COL1=x;
	DROP _NAME_;
RUN;

/* Analysis */
PROC GLM DATA=Adata OUTSTAT=STAT NOPRINT; 
	BY &by MEASURE;
	CLASS FILM RATER OCCAS;
	MODEL x=FILM RATER OCCAS FILM*RATER RATER*OCCAS FILM*OCCAS/SS3;
RUN;

DATA MS;SET STAT;
	MS=SS/DF;
	KEEP &BY MEASURE _SOURCE_ MS;
RUN;

DATA DF;SET STAT;
	KEEP &BY MEASURE _SOURCE_ DF;
RUN;

PROC TRANSPOSE DATA=MS OUT=MS1;
	BY  &BY MEASURE;
	ID _SOURCE_;
	VAR MS;
RUN;

PROC TRANSPOSE DATA=DF OUT=DF1 PREFIX=DF;
	BY &BY MEASURE;
	ID _SOURCE_;
	VAR DF;
RUN;

DATA MS_DF;
	SET MS1;
	SET DF1;
	BY &BY MEASURE;
	rename ERROR= MS_e; rename FILM=MS_p; rename RATER=MS_r; rename OCCAS=MS_o;
	rename FILM_RATER=MS_pr; rename RATER_OCCAS=MS_ro; rename FILM_OCCAS=MS_po;

	rename DFERROR=DF_e; rename DFFILM=DF_p; rename DFRATER=DF_r; rename DFOCCAS=DF_o;
	rename DFFILM_RATER=DF_pr; rename DFRATER_OCCAS=DF_ro; rename DFFILM_OCCAS=DF_po;

	n_p=DFFILM+1; n_r=DFRATER+1; n_o=DFOCCAS+1; 
RUN;

/* Calculation estimate and CI of ICC*/
DATA &out;SET MS_DF;
	numer1=(MS_p+MS_e-MS_pr-MS_po)/(n_r*n_o);
	denom1=MS_p/(n_r*n_o)+MS_r/(n_p*n_o)+MS_o/(n_p*n_r)+MS_pr*(1/n_o-1/(n_r*n_o)-1/(n_p*n_o))
       +MS_po*(1/n_r-1/(n_r*n_o)-1/(n_p*n_r))+MS_ro*(1/n_p-1/(n_p*n_o)-1/(n_p*n_r))
	   +MS_e*(1+1/(n_r*n_o)+1/(n_p*n_o)+1/(n_p*n_r)-1/n_p-1/n_r-1/n_o);
	ICC=numer1/denom1;         /* estimate of ICC*/
	Keep &BY MEASURE ICC;
RUN;
%MEND analyze;

%Macro boot(Odata=, Nsamples=, random=);/* uniform bootstrap resampling */

options nonotes;

%let alpha=0.05;

%global _nobs;
Data _null_;
      call symput('_nobs',trim(left(put(_nobs,12.))));
      if 0 then set &Odata nobs=_nobs;
      stop;
Run;

/*** Generate the bootstrap data   ***/
Data BOOTDATA;/*view=BOOTDATA*/
     * drop  _i; 
      do _sample_=1 to &Nsamples;
         do _i=1 to &_nobs;
               _j=ceil(ranuni(&random)*&_nobs);    /* choose a cell */
               set &Odata point=_j;output;
         end;
      end;
      stop;
Run;

/*** Calculate point and confidence interval estimates for ICC from original data ***/
%analyze(Odata=&Odata, out=_ACTUALL_, by= );

DATA _NULL_;
	SET _ACTUALL_;
	CALL symput('estimate',ICC);
RUN;    /*%Put what in estiamte: &estimate;*/


/*** Calculate ICC from each bootstrapped sample ***/
%analyze(Odata=BOOTDATA, out=BOOTDIST, by=_sample_);

PROC SORT data=BOOTDIST; By ICC; RUN;

DATA BOOTDIST;
	SET BOOTDIST;
	e_b=ICC-&estimate;

	If ICC > &estimate Then indicator=1;
	Else indicator=0;
RUN;

/*** Calculate Lower and Upper limit of confidence interval ***/
/* 1-The first percentile method (Efron)  p41 eq3.3 */
/* 2-The Second percentile method (Hall)  p42 eq3.4 */

PROC UNIVARIATE DATA=BOOTDIST NOPRINT;
     VAR ICC e_b;
     OUTPUT OUT=CI1_2
            N=n pctlpre=P_ICC P_E_b pctlpts=2.5,97.5, 5;  
RUN;

DATA CI1_2;
	SET CI1_2;
	llmt2=&estimate-P_E_b97_5;
	ulmt2=&estimate-P_E_b2_5;
	Lrho2=&estimate-P_E_b5;

	rename P_ICC2_5=llmt1 P_ICC97_5=ulmt1  P_ICC5=Lrho1 ;

	Keep P_ICC2_5 P_ICC97_5 llmt2 ulmt2 P_ICC5 Lrho2;
RUN;

/* 3-Biascorrected percentile CI (Efron, 1981a) p47 eq3.7 */
PROC UNIVARIATE DATA=BOOTDIST NOPRINT; 
	Var indicator;
	Output out=z_0 mean=proportion; 
RUN;

%Global Fhi_l Fhi_u rho_l;
DATA z_0;
	SET z_0;
	pro=1-proportion;
	z_0=probit(pro);
	z_alpha_half=probit(1-&alpha/2);
	z_alpha=probit(1-&alpha);

	Fhi_l=PROBNORM(2*z_0-z_alpha_half)*100;
	Fhi_u=PROBNORM(2*z_0+z_alpha_half)*100;

	rho_l=PROBNORM(2*z_0-z_alpha)*100;

	IF Fhi_l < 1.0E-6 Then Fhi_l=0.00001;
	IF Fhi_u < 1.0E-6 Then Fhi_u=0.00001;

	IF rho_l < 1.0E-6 Then rho_l=0.00001;

	CALL symput('Fhi_l', Fhi_l);
	CALL symput('Fhi_u', Fhi_u);
	CALL symput('rho_l', rho_l);
RUN;

PROC UNIVARIATE DATA=BOOTDIST NOPRINT;
     VAR ICC;
     OUTPUT OUT=CI3(rename=(P_l3=llmt3  P_u3=ulmt3  P_u3_1s=Lrho3))
            pctlpre=P_ pctlpts=&Fhi_l &Fhi_u &rho_l PCTLNAME=l3 u3 u3_1s;  
RUN;

/* estimate and CI of ICC */
DATA CI;
	SET _actuall_;
	SET CI1_2;
	SET CI3;

	Format ICC llmt1 ulmt1 length1 llmt2 ulmt2 length2 llmt3 ulmt3 length3  
               Lrho1 Lrho2 Lrho3 lth1 lth2 lth3 6.4;
/*two-sided CI length*/
	length1=ulmt1-llmt1;
	length2=ulmt2-llmt2;
	length3=ulmt3-llmt3;
/*one-sided CI length*/
	lth1=1-Lrho1;
	lth2=1-Lrho2;
	lth3=1-Lrho3;

	Keep Measure ICC llmt1 ulmt1 length1 /*llmt2 ulmt2 length2*/ llmt3 ulmt3 length3;
RUN;

PROC PRINT DATA=CI;
	TITLE "Estimated Intra-class Correlation Coefficient and CI from Bootstrapping";
RUN;

OPTIONS NOTES;
%Mend boot;




/* Read in data regarding measurement RRAVT1T2 */
DATA Xray; set sto.Xray_RRAVT1T2;
RUN;

%boot(Odata=Xray, Nsamples=10000, random=123);

