/*********************************************************************************************/
/* Gibbs Sampling                                                                            */
/* This macro is designed to estimate ICC and its CI for full 3-way random model with up to  */
/* 4 raters(coded as A,B,C,D) and 3 occasions (coded as 1,2,3)                               */
/* Argument discription: n_p, n_r, n_o are numbers of patient(film), raters and occasions    */
/*                       alpha, beta and eita are prior parameters;                          */
/*                       nrep is the length of chain and burnin is burnin period, burnin<nrep;*/
/*                       measurement is the response variable;                               */
/*                       seed is for generating random number;                               */
/*                       startValue is a binary number, if =0 then the starting values of    */
/*                          all random effects are set to be 1; if=1, deviation components   */
/*                          that comprise the Sum of Squares will be used.                   */
/* Usage: change Macro variable "location" to where original data set is saved.  The         */
/*        analysis results will also be saved there.                                         */
/*********************************************************************************************/
DM Editor 'clear log' editor; DM Editor 'clear output' editor;

options ls=125 ps=60 nodate nonumber;
%Let location=C:\Documents and Settings\etc\;
LIBNAME sto "&location";

%let startValue=1;
*options nomprint;
%Macro gibbs(alpha=, beta=, eita=, nrep=, burnin=, measurement=, seed=, startValue=);

PROC FORMAT;
  INVALUE  num  'A'=1 'B'=2 'C'=3 'D'=4 '1'=1 '2'=2 '3'=3;
RUN;

/* Read the original data                      */
DATA Xray; set sto.Xray_RRAVT1T2;
  *WHERE measurement="RRAVT1T2";
RUN;

PROC TRANSPOSE DATA=Xray OUT=Adata;
  BY MEASURE FILM;
RUN;

DATA Two; SET Adata;
  PATIENT=FILM;
  RATER=input(SUBSTR(_NAME_,1,1), num.);
  OCCAS=input(SUBSTR(_NAME_,2,1), num.);
  x    =COL1;
  DROP _NAME_ MEASURE FILM COL1;
  resid_wmu=.; resid_wp=.; resid_wr=.; resid_wo=.;resid_wpr=.; resid_wpo=.; resid_wro=.; resid=.;
RUN;

PROC SQL NOPRINT;
  SELECT count(distinct PATIENT) as n into: n_p
  FROM Two;
  SELECT count(distinct RATER)   as n into: n_r
  FROM Two;
  SELECT count(distinct OCCAS)   as n into: n_o
  FROM Two;
QUIT;

/***********************************************************************************/
/* Evaluate the initial values of the random effects of mu, p, r, o, pr, po and ro */
/***********************************************************************************/
proc sql;
	create table mu as 
	select mean(x) as mu
	from Two;
/*random effects of P*/
	create table p as
	select unique patient, mean(x)-mu as p
	from two, mu
	group by patient;
/*random effects of R*/
	create table r as
	select unique rater, mean(x)-mu as r
	from two, mu
	group by rater;
/*random effects of O*/
    create table o as
    select unique occas, mean(x)-mu as o
    from two, mu
    group by occas;
/*random effects of PR -- interaction between Patient and Rater*/
    create table pr as
    select unique two.patient, two.rater, mean(x)-p-r-mu as pr
    from two, mu, p, r
    where two.patient=p.patient and two.rater=r.rater
    group by two.patient, two.rater;
/*random effects of PO -- interaction between Patient and Occasion*/
    create table po as
    select unique two.patient, two.occas, mean(x)-p-o-mu as po
    from two, mu, p, o
    where two.patient=p.patient and two.occas=o.occas
    group by two.patient, two.occas;
/*random effects of RO -- interaction between Rater and Occasion*/
    create table ro as
    select unique two.rater, two.occas, mean(x)-r-o-mu as ro
    from two, mu, r, o
    where two.rater=r.rater and two.occas=o.occas
    group by two.rater, two.occas;
quit;

Options nonotes;

PROC IML;
/*************   Starting values of random terms        **************************************/
/* If startValue is 0, then the starting values of all random effects are set to be 1;       */
/* Otherwise, they are set to be deviation components that comprise the their Sum of Squares.*/
/*********************************************************************************************/
 %If %bquote(&startValue)=  OR &startValue=0 %Then %Do;
    mu=0;
	pa=repeat(1,&n_p,1);    ra=repeat(1,&n_r,1);    oc=repeat(1,&n_o,1); 
	pr=repeat(1,&n_p,&n_r);	po=repeat(1,&n_p,&n_o);	ro=repeat(1,&n_r,&n_o);
 %End;
 %Else %Do; 
	Use mu;	Read all Var{mu} into mu; Close mu;	Use P ; Read all Var{p} into PA;  Close P;
	Use R ; Read all Var{r} into RA;  Close R;	Use O ; Read all Var{o} into OC;  Close O;
	Use PR ; Read all Var{pr} into pr; pr=shape(pr,&n_p,&n_r); Close PR;
	Use PO ; Read all Var{po} into po; po=shape(po,&n_p,&n_o); Close PO;
	Use RO ; Read all Var{ro} into ro; ro=shape(ro,&n_r,&n_o); Close RO;
 %End;
	tau_p=1; tau_r=1; tau_o=1; tau=1; tau_pr=1; tau_po=1; tau_ro=1;/**/
	n_p=&n_p;	n_r=&n_r;  n_o=&n_o;

/*********************************  start Gibbs sampling  *******************************/
Do rep=1  to &nrep;

	var_p =1/tau_p; 	var_r =1/tau_r; 	var_o =1/tau_o; 	var   =1/tau;
	var_pr=1/tau_pr;	var_po=1/tau_po;	var_ro=1/tau_ro;
	rho=var_p/(var_p + var_r + var_o + var_pr + var_po + var_ro + var);

	Use TWO; Read all into Rdata; Close two;   /* Read in original data from data set TWO */
	mattrib Rdata colname=({'subj' 'rate' 'occa' 'x' 'x_mu' 'x_p' 'x_r' 'x_o' 'x_pr' 'x_po' 'x_ro' 'resid'});

/****************************** 1. Update Variance Components **********************************/
/****** tau ---  alpha=&alpha+n_p*n_r*n_o/2;  beta =1/(1/&beta+ssq(Rdata[,'resid'])/2);  */
	Do i=1  to n_p;
		 Do j=1  to n_r;
		 	Do k=1 to n_o;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'resid']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu-pa[i]-ra[j]-oc[k]-pr[i,j]-po[i,k]-ro[j,k]; 
	 		End;
		 End;
	End;
	tau=(1/(1/&beta+ssq(Rdata[,'resid'])/2))*RANGAM(&SEED,&alpha+n_p*n_r*n_o/2); 

/****** tau_p --- alpha=&alpha+n_p/2;  beta =1/(1/&beta +ssq(pa)/2);   */
	tau_p=(1/(1/&beta +ssq(pa)/2))*RANGAM(&SEED,&alpha+n_p/2); 

/****** tau_r --- alpha=&alpha+n_r/2;  beta =1/(1/&beta +ssq(ra)/2);   */
	tau_r=(1/(1/&beta +ssq(ra)/2))*RANGAM(&SEED,&alpha+n_r/2); 

/****** tau_o --- alpha=&alpha+n_o/2;  beta =1/(1/&beta +ssq(oc)/2);   */
	tau_o=(1/(1/&beta +ssq(oc)/2))*RANGAM(&SEED,&alpha+n_o/2); 

/****** tau_pr -- alpha=&alpha+n_p*n_r/2; beta =1/(1/&beta +ssq(pr)/2);*/
	tau_pr=(1/(1/&beta +ssq(pr)/2))*RANGAM(&SEED,&alpha+n_p*n_r/2); 

/****** tau_po -- alpha=&alpha+n_p*n_o/2; beta =1/(1/&beta +ssq(po)/2);*/
	tau_po=(1/(1/&beta +ssq(po)/2))*RANGAM(&SEED,&alpha+n_p*n_o/2);

/****** tau_ro -- alpha=&alpha+n_r*n_o/2; beta =1/(1/&beta +ssq(ro)/2);*/
	tau_ro=(1/(1/&beta +ssq(ro)/2))*RANGAM(&SEED,&alpha+n_r*n_o/2);

/************************** 2. Update Random Effects  **************************************/
/****** Pa_i ---  Mean_p= tau*sum(Rdata[loc(Rdata[ ,'subj']=i), 'x_p'])/(n_r*n_o*tau+tau_p);
				  v_p   = 1/(n_r*n_o*tau+tau_p);    ***/
	Do i=1 to n_p; 
		Do j=1 to n_r;
			Do k=1 to n_o;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_p']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu     -ra[j]-oc[k]-pr[i,j]-po[i,k]-ro[j,k]; 
			End; 
	 	End;
		pa[i]=tau*sum(Rdata[loc(Rdata[ ,'subj']=i), 'x_p'])/(n_r*n_o*tau+tau_p)
				+sqrt(1/(n_r*n_o*tau+tau_p))*NORMAL(&seed); 
	End;

/****** Ra_j ---  Mean_r= tau*sum(Rdata[loc(Rdata[ ,'rate']=j), 'x_r'])/(n_p*n_o*tau+tau_r);
				  v_r   = 1/(n_p*n_o*tau+tau_r);   ***/
	Do j=1 to n_r; 
		Do i=1 to n_p;
			Do k=1 to n_o;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_r']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu-pa[i]     -oc[k]-pr[i,j]-po[i,k]-ro[j,k]; 
	  		End; 
	 	End;
		ra[j]=tau*sum(Rdata[loc(Rdata[ ,'rate']=j), 'x_r'])/(n_p*n_o*tau+tau_r)
				+sqrt(1/(n_p*n_o*tau+tau_r))*NORMAL(&seed); 
	End;

/****** Oc_k --- Mean_o= tau*sum(Rdata[loc(Rdata[ ,'occa']=k), 'x_o'])/(n_p*n_r*tau+tau_o);
				 v_o   = 1/(n_p*n_r*tau+tau_o);  ***/
	Do k=1 to n_o; 
		Do i=1 to n_p;
			Do j=1 to n_r;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_o']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu-pa[i]-ra[j]     -pr[i,j]-po[i,k]-ro[j,k]; 
	 		End; 
	 	End;
		oc[k]=tau*sum(Rdata[loc(Rdata[ ,'occa']=k), 'x_o'])/(n_p*n_r*tau+tau_o)
				+sqrt(1/(n_p*n_r*tau+tau_o))*NORMAL(&seed); 
	End;

/****** pr_ij --- Mean_pr= tau*sum(Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j), 'x_pr'])/(n_o*tau+tau_pr);
				  v_pr   = 1/(n_o*tau+tau_pr);  ***/
	Do i=1 to n_p; 
		Do j=1 to n_r;
			Do k=1 to n_o;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_pr']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu-pa[i]-ra[j]-oc[k]       -po[i,k]-ro[j,k]; 
			End;
			pr[i,j]=tau*sum(Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j),'x_pr'])/(n_o*tau+tau_pr)
					+sqrt(1/(n_o*tau+tau_pr))*NORMAL(&seed); 
	 	End;
	End;

/****** po_ik --- Mean_po= tau*sum(Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'occa']=k),'x_po'])/(n_r*tau+tau_po);
				  v_po   = 1/(n_r*tau+tau_po);  ***/
	Do i=1 to n_p; 
		Do k=1 to n_o;
			Do j=1 to n_r;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_po']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu-pa[i]-ra[j]-oc[k]-pr[i,j]      -ro[j,k]; 
	 		End;
			po[i,k]=tau*sum(Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'occa']=k),'x_po'])/(n_r*tau+tau_po)
					+sqrt(1/(n_r*tau+tau_po))*NORMAL(&seed); 
	 	End;
	End;

/****** ro_jk --- Mean_ro= tau*sum(Rdata[loc(Rdata[ ,'rate']=j & Rdata[ ,'occa']=k),'x_ro'])/(n_p*tau+tau_ro);
				  v_ro   = 1/(n_p*tau+tau_ro);  ***/
	Do j=1 to n_r; 
		Do k=1 to n_o;
			Do i=1 to n_p;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_ro']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					-mu-pa[i]-ra[j]-oc[k]-pr[i,j]-po[i,k]     ; 
			End;
			ro[j,k]=tau*sum(Rdata[loc(Rdata[ ,'rate']=j & Rdata[ ,'occa']=k),'x_ro'])/(n_p*tau+tau_ro)
					+sqrt(1/(n_p*tau+tau_ro))*NORMAL(&seed); 
	 	End;
	End;

/****** mu   --- mean_mu=tau*sum(Rdata[ ,'x_mu'])/(1/&eita+tau*n_p*n_r*n_o);
				 v_mu   =1/(1/&eita+tau*n_p*n_r*n_o);***/
	Do i=1  to n_p;
		 Do j=1  to n_r;
		 	Do k=1 to n_o;
				Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x_mu']
				=Rdata[loc(Rdata[ ,'subj']=i & Rdata[ ,'rate']=j & Rdata[ ,'occa']=k) ,'x']
					   -pa[i]-ra[j]-oc[k]-pr[i,j]-po[i,k]-ro[j,k]; 
			End;
		 End;
	End;
	mu=tau*sum(Rdata[ ,'x_mu'])/(1/&eita+tau*n_p*n_r*n_o) 
		+sqrt(1/(1/&eita+tau*n_p*n_r*n_o))*NORMAL(&seed);
/*********************** End of one iteration of updating parameters  ********************/

	mattrib outMatrix colname={'var_p' 'var_r' 'var_o' 'var_pr' 'var_po' 'var_ro' 'var' 'rho'};
	outMatrix= outMatrix // (var_p || var_r || var_o || var_pr || var_po || var_ro || var || rho);

    *print mu PA RA OC PR PO RO;
	*print tau tau_p tau_r tau_o tau_pr tau_po tau_ro;
End;

create result from outMatrix [colname={'var_p' 'var_r' 'var_o' 'var_pr' 'var_po' 'var_ro' 'var' 'rho'}]; 
   append from outMatrix;
close result;

quit; /* End of IML proc */

/* Export the results of Markov Chain for further use*/
Data sto.&measurement._I&nrep._sd&seed._start&startValue;
	Set result;
Run;

/* *** discard the burn-in period */
Data trunResult;
	Set result;
	If _N_ > &burnin and  _N_ <= &nrep;
Run;

/* estimation of ICC and CI*/
Proc univariate data=trunResult noprint;
	Var rho;
	Output out=estimate median=ICC_md mean=ICC pctlpre=CI pctlpts=2.5,97.5;
Run;

Data estimate;
	Set estimate;
	measure=SUBSTR("&measurement", 1,8);
	n=&nrep;	
	m=&burnin;
	ICC_estimate=round(ICC_md,0.01);
	lowerCI=round(CI2_5, 0.01);
	UpperCI=round(CI97_5,0.01);
	CI_length=round(CI97_5-CI2_5, 0.01);
	Drop ICC_md ICC  CI2_5  CI97_5;
run;

Proc Print data=estimate; 
	TITLE1 "Estimated Intra-class Correlation Coefficient and CI from MCMC";
	TITLE2 "using a single chain of random seed &seed, length &nrep and burnin &burnin";
Run;

%Mend; 

/* For example, to calculate the estimate and CI of ICC for meansurement T1T12VER using a chain 
of length 5000 and burn-in 3500, call the macro like this */
%gibbs(alpha=0.0001, beta=100000000, eita=10000, nrep=5000, burnin=3500, 
       measurement=RRAVT1T2, seed=12300530, startValue=1);

/* If more chains are needed, call the macro with a different seed and combine the results in data
set "trunResult" with that from previous call, then use the combined results to do the estimation as
done in step "estimation of ICC and CI" in the macro*/
%gibbs(alpha=0.0001, beta=100000000, eita=10000, nrep=5000, burnin=3500, 
       measurement=RRAVT1T2, seed=123, startValue=1);
